@@ -20,20 +20,20 @@ class YaraCommentEnabledClass:
 
     def add_comment(self, comment, position="inline"):
         """
-        Add a comment entry
+        Add a comment entry (appends to above and below, replaces inline)
 
         Args:
             comment (str): the comment
             position (str): the position of the comment relative to the entry
         """
         if position == "above":
-            self.yara_comment.above = comment
+            self.yara_comment.above.append(comment)
 
         elif position == "inline":
             self.yara_comment.inline = comment
 
         elif position == "below":
-            self.yara_comment.below = comment
+            self.yara_comment.below.append(comment)
 
     def build_comments(self, raw, whitespace="    "):
         """
@@ -47,23 +47,25 @@ class YaraCommentEnabledClass:
             str: the entry with comments built around it
         """
         if self.yara_comment.above:
-            raw = "// %s\n%s%s%s" % (
-                self.yara_comment.above,
-                whitespace,
-                whitespace,
-                raw,
-            )
+            for above_comment in reversed(self.yara_comment.above):
+                raw = "// %s\n%s%s%s" % (
+                    above_comment,
+                    whitespace,
+                    whitespace,
+                    raw,
+                )
 
         if self.yara_comment.inline:
             raw = "%s // %s" % (raw, self.yara_comment.inline)
 
         if self.yara_comment.below:
-            raw = "%s\n%s%s// %s" % (
-                raw,
-                whitespace,
-                whitespace,
-                self.yara_comment.below,
-            )
+            for below_comment in self.yara_comment.below:
+                raw = "%s\n%s%s// %s" % (
+                    raw,
+                    whitespace,
+                    whitespace,
+                    below_comment,
+                )
 
         return raw
 
@@ -74,18 +76,18 @@ class YaraComment:
     Can be applied to YaraMetaEntry and YaraString
 
     Attributes:
-        above (str): comment located a line above the entry
+        above (list): comment(s) located a line above the entry
         inline (str): comment located on the same line after the entry
-        below (str): comment located a line below the entry
+        below (list): comment(s) located a line below the entry
     """
 
     def __init__(self):
         """
         Constructor for YaraComment
         """
-        self.above = None
+        self.above = []
         self.inline = None
-        self.below = None
+        self.below = []
 
     def get_yara_comment(self):
         """
@@ -164,7 +166,7 @@ class YaraMetaEntry(YaraCommentEnabledClass):
         elif self.meta_type == "int":
             self.raw_meta_entry = "%s = %d" % (self.name, self.value)
 
-        if self.meta_type == "bool":
+        elif self.meta_type == "bool":
             if self.value:
                 self.raw_meta_entry = "%s = true" % self.name
 
@@ -234,7 +236,7 @@ class YaraMeta:
         logger (Logger): the logger for this class
     """
 
-    def __init__(self):
+    def __init__(self, logger=None):
         """
         Constructor for YaraMeta
         """
@@ -242,7 +244,7 @@ class YaraMeta:
         self.raw_meta = []
         self.number_of_meta_entries = 0
         self.valid_meta_types = ["text", "int", "bool"]
-        self.logger = logging.getLogger(__name__)
+        self.logger = logger or logging.getLogger(__name__)
 
     def add_meta(self, name, value, meta_type="text"):
         """
@@ -783,11 +785,7 @@ class YaraRule:
         self.condition = YaraCondition()
         self.imports = YaraImports()
         self.tags = YaraTags()
-
-        if logger:
-            self.logger = logger
-        else:
-            self.logger = logging.getLogger(__name__)
+        self.logger = logger or logging.getLogger(__name__)
 
     def build_rule_header(self, rule):
         """
@@ -799,11 +797,16 @@ class YaraRule:
         Returns:
             str: string of the built rule with added rule header
         """
+
         if self.imports.has_imports():
+            self.logger.debug("Building imports for %s...", self.rule_name)
+
             self.imports.build_imports()
             rule += "%s\n" % self.imports.raw_imports
 
         if self.tags.has_tags():
+            self.logger.debug("Building tags for %s...", self.rule_name)
+
             self.tags.build_tags()
             rule += "rule %s : %s {\n" % (self.rule_name, self.tags.raw_tags)
         else:
@@ -885,14 +888,18 @@ class YaraRule:
                 '"{0}" has no raw_condition, cannot build rule'.format(self.rule_name)
             )
 
+        self.logger.debug("Building rule header for %s...", self.rule_name)
         self.raw_rule = self.build_rule_header(self.raw_rule)
 
         if self.meta.number_of_meta_entries > 0:
+            self.logger.debug("Building meta section for %s...", self.rule_name)
             self.raw_rule = self.build_rule_meta_section(self.raw_rule)
 
         if self.strings.number_of_strings > 0:
+            self.logger.debug("Building strings section for %s...", self.rule_name)
             self.raw_rule = self.build_rule_strings_section(self.raw_rule)
 
+        self.logger.debug("Building condition section for %s...", self.rule_name)
         self.raw_rule = self.build_rule_condition_section(self.raw_rule)
 
         return self.raw_rule
